from si_fab import all as pdk
from si_fab.components.heater.simulation.simulate import simulate_heater
from si_fab.components.heater.pcell.cell import r_sheet
import pylab as plt
import numpy as np
import os

name = "exercise_2"
results_array = []
length = 100
dphi = np.pi / 2.0  # V
dv = 1.0
P_pi = 30e-3  # W
p_pi_sq = r_sheet * P_pi
heater_width = 0.6

heater_length = np.pi * dv**2 * 2 * heater_width / (p_pi_sq * dphi)

ht = pdk.HeatedWaveguide(
    heater_width=0.6,
    heater_offset=1.0,
    m1_extension=0.2,
    m1_length=3.0,
)
ht_lv = ht.Layout(shape=[(0.0, 0.0), (heater_length, 0.0)])

ht.CircuitModel(p_pi_sq=p_pi_sq)

results = simulate_heater(
    cell=ht,
    v_start=0,
    v_end=1,
    nsteps=500,
    center_wavelength=1.5,
    debug=False,
)
times = results.timesteps
results_array.append(results)


def phase_unwrap_normalize(results):
    unwrapped = np.unwrap(np.angle(results))
    return (unwrapped - unwrapped[0]) / (np.pi)


signals = ["optical_in", "gnd", "vsrc", "optical_out"]
processes = [np.real, np.real, np.real, phase_unwrap_normalize]
ylabels = ["V", "V", "V", r"[rad/$\pi$]"]

fig, axs = plt.subplots(nrows=len(signals), ncols=1, figsize=(6, 8))
for axes, process, signal, ylabel in zip(axs, processes, signals, ylabels):
    data = process(results[signal])
    axes.set_title(signal)
    axes.plot(times, data, label=f"length: {length}")
    axes.set_ylabel(ylabel)
    axes.set_xlabel("time [s]")

plt.tight_layout()
fig.savefig(os.path.join(f"{name}.png"), bbox_inches="tight")
plt.show()
plt.close(fig)
